#
#  The OpenDiamond Platform for Interactive Search
#
# SPDX-License-Identifier: EPL-1.0
#
#  Copyright (c) 2021 Carnegie Mellon University
#  All rights reserved.
#
#  This software is distributed under the terms of the Eclipse Public
#  License, Version 1.0 which can be found in LICENSES/EPL-1.0.
#  ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS SOFTWARE CONSTITUTES
#  RECIPIENT'S ACCEPTANCE OF THIS AGREEMENT
#
"""Functions to assist with handling application/x-diamond-scope files"""

import click

from .scope_generate import generate
from .scope_import import import_, install, uninstall
from .scope_options import CONTEXT_SETTINGS
from .scope_verify import verify


@click.group("scope", context_settings=CONTEXT_SETTINGS)
def cli():
    """Scope handling related functions"""


cli.add_command(generate)
cli.add_command(import_)
cli.add_command(install)
cli.add_command(uninstall)
cli.add_command(verify)



#
#  The OpenDiamond Platform for Interactive Search
#
# SPDX-FileCopyrightText: 2021 Carnegie Mellon University
# SPDX-License-Identifier: EPL-1.0
#

import pytest
from click.testing import CliRunner


@pytest.fixture(scope="module")
def isolated_runner():
    runner = CliRunner()
    with runner.isolated_filesystem():
        yield runner



#
#  The OpenDiamond Platform for Interactive Search
#
# SPDX-License-Identifier: EPL-1.0
#
#  Copyright (c) 2021 Carnegie Mellon University
#  All rights reserved.
#
#  This software is distributed under the terms of the Eclipse Public
#  License, Version 1.0 which can be found in LICENSES/EPL-1.0.
#  ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS SOFTWARE CONSTITUTES
#  RECIPIENT'S ACCEPTANCE OF THIS AGREEMENT
#
"""Functions to assist with handling application/x-diamond-scope files"""

import os
import subprocess
import sys
import tempfile
from datetime import datetime
from pathlib import Path

import click
from dateutil.tz import tzutc

from ..scope import ScopeCookie, ScopeCookieExpired, ScopeError
from .scope_options import CONTEXT_SETTINGS, prefix_option

# diamond scope mime.xml and .desktop related paths
SCOPE_CONFIG_FILES = [
    (
        Path("share", "mime", "packages", "application-x-diamond-scope.xml"),
        """\
<?xml version="1.0" encoding="utf-8"?>
<mime-info xmlns="http://www.freedesktop.org/standards/shared-mime-info">
    <mime-type type="application/x-diamond-scope">
        <comment>Diamond scope</comment>
        <magic>
            <match type="string" offset="0" value="-----BEGIN OPENDIAMOND SCOPECOOKIE"/>
        </magic>
        <glob pattern="*.scope"/>
    </mime-type>
</mime-info>
""",
    ),
    (
        Path("share", "applications", "opendiamond-scope.desktop"),
        """\
[Desktop Entry]
Version=1.0
Type=Application
MimeType=application/x-diamond-scope;
Name=Diamond Scope Handler
Exec=opendiamond-scope import %F
NoDisplay=true
""",
    ),
]


@click.command("import", context_settings=CONTEXT_SETTINGS)
@click.argument(
    "scope",
    nargs=-1,
    required=True,
    type=click.Path(exists=True, dir_okay=False, path_type=Path),
)
def import_(scope):
    """Combine OpenDiamond scopes into %app_dir%/NEWSCOPE.

    Accepts one or more scope files as arguments. Normally called as a handler
    when downloading files with the application/x-diamond-scope mime type.
    """
    app_dir = Path(click.get_app_dir("diamond", force_posix=True))
    if not app_dir.is_dir():
        click.echo(
            f"{click.format_filename(app_dir)} directory does not exist.", err=True
        )
        sys.exit(1)

    # make sure application directory is not publically readable
    if app_dir.stat().st_mode & 0o077:
        click.echo(f"Making sure {app_dir} is not publically readable.", err=True)
        app_dir.chmod(0o700)

    now = datetime.now(tzutc())

    newscope_fd, newscope_tmp = tempfile.mkstemp(dir=app_dir, text=True)
    with os.fdopen(newscope_fd, "w") as newscope_fh:
        for scopefile in scope:
            # validate that the scopes are correctly formatted and not expired.
            # we can't validate if the signatures/servers/scopes are correct.
            try:
                megacookie = scopefile.read_text()
                for cookie in ScopeCookie.split(megacookie):
                    parsed = ScopeCookie.parse(cookie)
                    if parsed.expires < now:
                        raise ScopeCookieExpired(f"Cookie expired on {parsed.expires}")

                    # passed validation, append to NEWSCOPE
                    newscope_fh.write(cookie)
            except ScopeError as exception:
                click.echo(f"Unabled to read {scopefile}: {str(exception)}", err=True)

    # and replace the NEWSCOPE file with the new scope
    Path(newscope_tmp).replace(app_dir / "NEWSCOPE")


@click.command(context_settings=CONTEXT_SETTINGS)
@prefix_option
def install(prefix):
    """Installs mime-type handler for application/x-diamond-scope files."""
    for path, content in SCOPE_CONFIG_FILES:
        filepath = prefix / path
        dirpath = filepath.parent

        click.echo(f"Making sure {dirpath} exists")
        dirpath.mkdir(parents=True, exist_ok=True)

        click.echo(f"Creating {filepath}")
        filepath.write_text(content)

    update_mime_desktop_databases(prefix)


@click.command(context_settings=CONTEXT_SETTINGS)
@prefix_option
def uninstall(prefix):
    """Removes installed mime-type handler for application/x-diamond-scope files."""
    for path, _ in SCOPE_CONFIG_FILES:
        filepath = prefix / path
        click.echo(f"Removing {filepath}")
        filepath.unlink(missing_ok=True)

    update_mime_desktop_databases(prefix)


def update_mime_desktop_databases(prefix):
    """Updates mime-type and .desktop databases"""
    mime_path = prefix / "share" / "mime"
    if mime_path.exists():
        subprocess.run(["update-mime-database", mime_path], check=False)

    app_path = prefix / "share" / "applications"
    if app_path.exists():
        subprocess.run(["update-desktop-database", app_path], check=False)



#
#  The OpenDiamond Platform for Interactive Search
#
# SPDX-FileCopyrightText: 2021 Carnegie Mellon University
# SPDX-License-Identifier: EPL-1.0
#

from click.testing import CliRunner

from opendiamond.console.scope import cli

from .test_cookies import KeyPair


# Test opendiamond-scope generate
def test_generate_required_servers_option():
    runner = CliRunner()

    result = runner.invoke(cli, ["generate"])
    assert result.exit_code == 2
    assert "Missing option" in result.output


def test_generate_keyfile_option(isolated_runner):
    # try with non-existing key file
    opts = ["generate", "-s", "diamond.test", "-k", "key.pem", "-v"]
    result = isolated_runner.invoke(cli, opts)
    assert result.exit_code == 2
    assert "No such file" in result.output


def test_generate_invalid_keyfile_option(isolated_runner):
    # try with invalid key file
    opts = ["generate", "-s", "diamond.test", "-k", "key.pem", "-v"]
    with open("key.pem", "w") as keyfile:
        keyfile.write("")

    result = isolated_runner.invoke(cli, opts)
    assert result.exit_code == 1
    assert result.exception
    assert "Unable to read private key" in str(result.exception)


def test_generate_valid_keyfile_option(isolated_runner):
    # try with valid key file
    opts = ["generate", "-s", "diamond.test", "-k", "key.pem", "-v"]
    with open("key.pem", "w") as keyfile:
        keyfile.write(KeyPair.valid[0].key)

    result = isolated_runner.invoke(cli, opts)
    assert result.exit_code == 0
    assert "Servers: diamond.test" in result.output



#
#  The OpenDiamond Platform for Interactive Search
#
# SPDX-FileCopyrightText: 2009-2021 Carnegie Mellon University
# SPDX-License-Identifier: EPL-1.0
#

import base64
import binascii
import textwrap
import unittest
import uuid
from datetime import datetime, timedelta

from dateutil.tz import tzutc
from M2Crypto import EVP

from opendiamond.scope import ScopeCookie, ScopeError


# unittest uses Java-style naming conventions
# pylint: disable=invalid-name
class KeyPair:
    def __init__(self, key, cert):
        self.key = key
        self.cert = cert


KeyPair.valid = (
    KeyPair(
        """-----BEGIN RSA PRIVATE KEY-----
MIICXQIBAAKBgQDdMM19TmEBB/lwA5lym3nHwJ+xTL5pPg7F9JXzOs17TsaRI1Ot
8+r+PLGp51DPcfiTUrw6SaSbxdyVSNEavsHqh5WY4KW8SJp01rCke87utYgVIqPO
8ohikTHsOUzZ2FlRe2oHJ58kxEbmkr1/LQHM6CLd3Nq5rXW5AWVS/6sUzQIDAQAB
AoGAAfWPNGdv04cDDhtyDgJTi1Ho+DSSUTNUBSvb8ioYrNlvR9TUxmstLzBRcIxU
emnrhj6K3UdOtnSDtizNcVu0Mv/mJWiI8zdQ95mIQuGWJ65t27CAFttUEj2BwWwh
SjMK+6LIZyANoBU5Z1A7Tc/2OusnLhxE0jscWX2W9qivmr8CQQDqdvClupAN5a3i
eP0p5XVMsRmSJEaXXci2c2wMsbDWzFTiIHXLO15DzV7/A1NTA0EbNyBMd8Sca0OM
5W6Trty/AkEA8YG/FQ4fGjY/WNOVv0OKWdQQWpLHdI48jHf41oSGIXFYbg1peEhf
mEttuA7S33zx8Xry8on8X4DSiF1ie03VcwJBAKVKuJh9j7TcaNuyI/f3Vdx9NeO7
QzDO4wMNF+1TD7I+VtEbxS9DaU8vZ3ssYv+w2aNQF6sQ/ECxteuMi4n9yo0CQELe
a+wGhvIZBx0vKI1WxB3vK8AIWBWbtIQoS8wJ0aG84oFGUWeukll2uSB3akfbLppw
MoiZUDmZT7HWOYhumqUCQQCg0trsk9ymbPGckwcmfgB9CPFM8I90FPvunsSAGLL9
8hS0EClVedNWOJjTqO9SrlZcG6VVc5Kx2MAbLgUFTsck
-----END RSA PRIVATE KEY-----""",
        """-----BEGIN CERTIFICATE-----
MIIBojCCAQ2gAwIBAgIETdxa4TALBgkqhkiG9w0BAQUwADAeFw0xMTA1MjUwMTI2
NThaFw0zODEwMTAwMTI3MDZaMAAwgZ0wCwYJKoZIhvcNAQEBA4GNADCBiQKBgQDd
MM19TmEBB/lwA5lym3nHwJ+xTL5pPg7F9JXzOs17TsaRI1Ot8+r+PLGp51DPcfiT
Urw6SaSbxdyVSNEavsHqh5WY4KW8SJp01rCke87utYgVIqPO8ohikTHsOUzZ2FlR
e2oHJ58kxEbmkr1/LQHM6CLd3Nq5rXW5AWVS/6sUzQIDAQABoy8wLTAMBgNVHRMB
Af8EAjAAMB0GA1UdDgQWBBTy8/ILrYkeKTF1OGIox7oB7FoARzALBgkqhkiG9w0B
AQUDgYEAVR52PU6lCIw6pEMxSiXBNiIpONlatg3uxkOoeCg0hj0cjxVFys0RGyRa
S6pNCs64IwITNT/VtGmgr+uaLXLpNjqPtOdQaPLjtsSN1qgUJDNSf6bA7TgK2gdm
bTWRjkoonfKvs+VSz591rUuGd1d/FS8cwULQ93uk7eGwIRNUhcw=
-----END CERTIFICATE-----""",
    ),
    KeyPair(
        """-----BEGIN RSA PRIVATE KEY-----
MIICXgIBAAKBgQDEt1JQ7dG54DYYDHLeFg7jrQo/Zl/EpD6AseG8935q4L3ITyDR
kVtNFJCfX9TXuJ2FvTle9Oy9qtuXtpe1ABIEpUBPLHd98NUYj9Shr5juA/ljdk+4
r2F6H0wngz9Zo9HS8Gqx0pAXVn5jGZlGZesSDIskkW33WzRGQtcNminy7wIDAQAB
AoGATcJxhZAcoS8h8uuo1GU/yXvzWxBqtt593n1yUDI9BJ6GJpIw1OZygsuoR3eo
OQW/LsiXzxNeKbNKTnRdD61RcVYhEr9jA4gObgAv/B8DUJQVGSRzDF9Ih6yHIiyO
uwLfunNPLfaHcWBYQhPjFdihdJxOv5bBEDCjmQy1ZBnJVrECQQDImT8XXkpSC7rE
7D7LcvbsDfwPvu1WydOCdcxPCQDH3eBdQ2rII52di4uge8yrXAqfQTOsyMd01unR
UigWmhgbAkEA+wuSmQYe25SOml48C6lg5RIRydta2PL4PwzgpuR4WMYF8+ujPdXM
KMF8xpjsLvlio5YSb+n+xSTW/thuEeLlvQJBAKxHmPzb2R5/vmbzsraOROzU0d9A
DZwU+Bc6Tj6ur8H3l4LhrKq4k9xDhaZNzKh7AxBlMhk50rtQ/Dzuv0kyDHMCQQCr
jYf7fDxAD2+3u6fKzE+Dmmp/h5+4W6ka5QDr63r2JzRQMHmYOu7N9xL+X+geXZtz
cI1e3weTzw4AjwQAww39AkEAlTfJTrThfU5ufCIjqi08DoAVAFk7pyJ+Av7kJM2w
e5IStKIud/4XbyY9XzfLsJ9mUwYOA+b6xdD5pr4vJLpCDA==
-----END RSA PRIVATE KEY-----""",
        """-----BEGIN CERTIFICATE-----
MIIBojCCAQ2gAwIBAgIETdxa9jALBgkqhkiG9w0BAQUwADAeFw0xMTA1MjUwMTI3
MTlaFw0zODEwMTAwMTI3MjFaMAAwgZ0wCwYJKoZIhvcNAQEBA4GNADCBiQKBgQDE
t1JQ7dG54DYYDHLeFg7jrQo/Zl/EpD6AseG8935q4L3ITyDRkVtNFJCfX9TXuJ2F
vTle9Oy9qtuXtpe1ABIEpUBPLHd98NUYj9Shr5juA/ljdk+4r2F6H0wngz9Zo9HS
8Gqx0pAXVn5jGZlGZesSDIskkW33WzRGQtcNminy7wIDAQABoy8wLTAMBgNVHRMB
Af8EAjAAMB0GA1UdDgQWBBTJtnPW4VRb50WBiU4NMdNfxBnigDALBgkqhkiG9w0B
AQUDgYEAQcTWROP9N2S6KJMSppQY/Rq5A12xeRx+gono4ENxYIMSiBu7Vu61sy3H
i50uG9R6mGmKYioAV+DQeFPB4lulgbbQtVamLWN/lDZNjJVbEyrJfr00NMJA6vfu
FJj5CeodcrHc5mcsI/4hcgfdlLlNhfWWWLdggMtUdfyYPQvhmQA=
-----END CERTIFICATE-----""",
    ),
)
KeyPair.expired = KeyPair(
    """-----BEGIN RSA PRIVATE KEY-----
MIICWwIBAAKBgQC3N1er9/MgEtsYK1E90lO4BNXV/fbJMNNhtUPxkcBaaUUayQmy
TEuoXMSOtCnzMcI5wwbd65lcG+lnnzmkLRMb+0UVKv4UWaytjbZhgqQlaMvCFhOl
uC/D7tXkru0NX8ZGEh2wjiX9nsNCluxpnwuXr0NcOD+dhPfu3tfXBz8pHQIDAQAB
AoGAGUDkUXSJeyoGL3hnCAup1PM7qzWIctZLSIw024LwvbWFXCoKL1yDUCdLQ3Uz
VA1PfWkzlNqgxpilTQ5eIlJBMA4p+2EB88CZCVHaP+o00grphonGZHwv5v6tp9Da
SgJgTYVEGrZ5OfCS4yRgcjlxW1vwcXiIZ3tXNxKFmS1zthECQQDPeCTLFyN+njNR
E1sIEmoP9coVVCqL93TjAdGtf+UJbDMu8vA/AIw/pDTynfpxB6wQ32u9lrYSQFb3
eTbGo9ZtAkEA4hLaFfCurlQIW1Ooy8GG3o/iz3lljBi0bepDsWnJ/yfhQzT0QPMk
a7ATaJeP6ICxFOfdCeu1IYvqzhQi6x2vcQJAAOB8H1OgtcdLZjtTtiwFwL2ENiTd
7SuFlfQLA9W3jRuk97zVIR8KeLZj9uaOCW5D3upi1TFO4bLd6zva7GoC3QJAP8BT
MZaym8RkquRXmEXVs5NdwWYZZb1dvBUwy6nqZYKoelxHeL1YCuoXPwpmcYlA5oVQ
BskqRfB/4Wc6RZUUcQJAbULR4fpNBEbyhuxxJpu7RBBZTbGho74Bu9tPBDOebYC7
lM9mFmU2rPGq92DwSnWSBRSv5bXKFv9QpM6pPcStRg==
-----END RSA PRIVATE KEY-----""",
    """-----BEGIN CERTIFICATE-----
MIIBojCCAQ2gAwIBAgIETdxbKTALBgkqhkiG9w0BAQUwADAeFw0xMTA1MjUwMTI4
MDlaFw0xMTA1MjYwMTI4MTBaMAAwgZ0wCwYJKoZIhvcNAQEBA4GNADCBiQKBgQC3
N1er9/MgEtsYK1E90lO4BNXV/fbJMNNhtUPxkcBaaUUayQmyTEuoXMSOtCnzMcI5
wwbd65lcG+lnnzmkLRMb+0UVKv4UWaytjbZhgqQlaMvCFhOluC/D7tXkru0NX8ZG
Eh2wjiX9nsNCluxpnwuXr0NcOD+dhPfu3tfXBz8pHQIDAQABoy8wLTAMBgNVHRMB
Af8EAjAAMB0GA1UdDgQWBBTzO1jx/RdQU+Lvxs3ZmgdrGgwvHzALBgkqhkiG9w0B
AQUDgYEAdyKOmlVyoqStjvTL5MiW7YKx3Yc76FS2ZqNgP6a5GPZAkBTE3EwP54f5
pkVdYwevLPmdxVZmLZcpPvh42lzBdXvbcifZDAPLEO3hbuPKW7ZbZc3QRJhWNs8Q
t1NDO6wIwEXCa50isAJzw080w70XXmdpam408tZWM7dpWb0qHss=
-----END CERTIFICATE-----""",
)


class _TestScope(unittest.TestCase):
    servers = ["a", "b", "c"]
    scopeurls = ["z"]
    expires = datetime.now(tz=tzutc()) + timedelta(days=1)
    key = KeyPair.valid[0].key
    cert = KeyPair.valid[1].cert + "\n" + KeyPair.valid[0].cert
    serverids = ["q", "b"]
    generate_exc = None
    parse_exc = None
    verify_exc = ScopeError

    def runTest(self):
        # Hack: if this is a base class rather than an actual test class,
        # return without doing anything.
        if self.__class__.__name__.startswith("_"):
            return

        if self.generate_exc:
            self.assertRaises(self.generate_exc, self.generate_cookie)
            return
        data = self.generate_cookie()

        if self.parse_exc:
            self.assertRaises(self.parse_exc, lambda: self.parse_cookie(data))
            return
        cookie = self.parse_cookie(data)

        if self.verify_exc:
            self.assertRaises(self.verify_exc, lambda: self.verify_cookie(cookie))
            return
        self.verify_cookie(cookie)

    def generate_cookie(self):
        return ScopeCookie.generate(
            self.servers, self.scopeurls, self.expires, self.key
        ).encode()

    def parse_cookie(self, data):
        return ScopeCookie.parse(data)

    def verify_cookie(self, cookie):
        cookie.verify(self.serverids, self.cert)


class TestCookieRoundTrip(_TestScope):
    """Ensure that we can generate a cookie and verify it."""

    verify_exc = None


class TestCookieBadGenerateTimezone(_TestScope):
    """Try to generate a cookie with a naive expiration time."""

    expires = datetime.now() + timedelta(days=1)
    generate_exc = ScopeError


class TestCookieExpired(_TestScope):
    """Try to verify an expired cookie."""

    expires = datetime.now(tz=tzutc()) - timedelta(seconds=5)


class TestCookieExpiredCert(_TestScope):
    """Try to verify a cookie with an expired certificate."""

    key = KeyPair.expired.key
    cert = KeyPair.expired.cert


class TestCookieMissingCert(_TestScope):
    """Try to verify a cookie with a missing certificate."""

    cert = KeyPair.valid[1].cert


class TestCookieBadServerName(_TestScope):
    """Try to verify a cookie with a mismatched server ID."""

    serverids = ["x"]


class TestCookieEmptyScopeList(_TestScope):
    """Verify a cookie with a zero-length scope list."""

    scopeurls = []
    verify_exc = None


class _TestHandGeneratedCookie(_TestScope):
    boundary_start = "-----BEGIN OPENDIAMOND SCOPECOOKIE-----\n"
    boundary_end = "-----END OPENDIAMOND SCOPECOOKIE-----\n"
    serial = str(uuid.uuid4())
    version = 1
    expires = (datetime.now(tz=tzutc()) + timedelta(days=1)).isoformat()
    parse_exc = ScopeError
    verify_exc = None
    modify_sig = staticmethod(lambda s: s)
    modify_base64 = staticmethod(lambda s: s)

    def generate_cookie(self):
        """Hand-generate a cookie."""
        headers = dict()
        if self.version is not None:
            headers["Version"] = self.version
        if self.serial is not None:
            headers["Serial"] = self.serial
        if self.expires is not None:
            headers["Expires"] = self.expires
        if self.serverids is not None:
            headers["Servers"] = ";".join(self.serverids)
        hdrbuf = "".join(f"{key}: {value}\n" for key, value in headers.items())
        data = hdrbuf + "\n" + "\n".join(self.scopeurls) + "\n"
        key = EVP.load_key_string(self.key.encode())
        key.sign_init()
        key.sign_update(data.encode())
        sig = key.sign_final()
        body = self.modify_sig(binascii.hexlify(sig).decode()) + "\n" + data
        b64 = self.modify_base64(base64.b64encode(body.encode()).decode())
        return self.boundary_start + textwrap.fill(b64, 64) + "\n" + self.boundary_end


class TestSuccessfulHandGeneratedCookie(_TestHandGeneratedCookie):
    """Ensure that hand-generated cookies can be verified."""

    parse_exc = None


class TestCookieBadBoundary(_TestHandGeneratedCookie):
    """Try to parse a cookie with a bad end boundary."""

    boundary_end = "-----END COOKIE-----\n"


class TestCookieBadVersion(_TestHandGeneratedCookie):
    """Try to parse a cookie with an unrecognized version."""

    version = 7


class TestCookieBadSerial(_TestHandGeneratedCookie):
    """Try to parse a cookie with a badly-formatted serial number."""

    serial = "1-2-3-4-5"


class TestCookieBadExpiration(_TestHandGeneratedCookie):
    """Try to parse a cookie with a badly-formatted expiration date."""

    expires = "no"


class TestCookieBadVerifyTimezone(_TestHandGeneratedCookie):
    """Try to parse a cookie with a naive expiration date."""

    expires = (datetime.now() + timedelta(days=1)).isoformat()


class TestCookieMissingExpiration(_TestHandGeneratedCookie):
    """Try to parse a cookie with a missing expiration date."""

    expires = None


class TestCookieMissingServer(_TestHandGeneratedCookie):
    """Try to parse a cookie with a missing server list."""

    serverids = None


class TestCookieMissingSerial(_TestHandGeneratedCookie):
    """Try to parse a cookie with a missing serial number."""

    serial = None


class TestCookieBadBase64(_TestHandGeneratedCookie):
    """Try to parse a cookie with invalid base64 encoding."""

    modify_base64 = staticmethod(lambda s: "*" + s[1:])


class TestCookieBadSignature(_TestHandGeneratedCookie):
    """Try to verify a cookie with an invalid signature."""

    modify_sig = staticmethod(lambda s: "331337" + s[6:])
    parse_exc = None
    verify_exc = ScopeError


if __name__ == "__main__":
    unittest.main()



#
#  The OpenDiamond Platform for Interactive Search
#
# SPDX-FileCopyrightText: 2021 Carnegie Mellon University
# SPDX-License-Identifier: EPL-1.0
#

from click.testing import CliRunner

from opendiamond.console.scope import cli


# Test opendiamond-scope import
def test_import_missing_scope_argument():
    runner = CliRunner()

    result = runner.invoke(cli, ["import"])
    assert result.exit_code == 2
    assert "Missing argument" in result.output


def test_import_missing_scope(isolated_runner):
    # try with non-existing scope file
    result = isolated_runner.invoke(cli, ["import", "scope"])
    assert result.exit_code == 2
    assert "File 'scope' does not exist" in result.output



#
#  The OpenDiamond Platform for Interactive Search
#
# SPDX-License-Identifier: EPL-1.0
#
#  Copyright (c) 2009-2021 Carnegie Mellon University
#  All rights reserved.
#
#  This software is distributed under the terms of the Eclipse Public
#  License, Version 1.0 which can be found in LICENSES/EPL-1.0.
#  ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS SOFTWARE CONSTITUTES
#  RECIPIENT'S ACCEPTANCE OF THIS AGREEMENT
#
"""CLI app to validate an OpenDiamond scope"""

from pathlib import Path

import click

from ..scope import ScopeCookie, ScopeError
from .scope_options import CONTEXT_SETTINGS


@click.command(context_settings=CONTEXT_SETTINGS)
@click.argument(
    "scope",
    required=True,
    type=click.Path(exists=True, dir_okay=False, path_type=Path),
)
@click.argument(
    "server",
    required=False,
)
@click.argument(
    "certificates",
    required=False,
    type=click.Path(exists=True, dir_okay=False, path_type=Path),
)
def verify(scope, server, certificates):
    """Show content of a scope file and optionally verify the signature."""
    # certificate validation is optional
    if not certificates:
        app_dir = Path(click.get_app_dir("diamond", force_posix=True))
        certificates = app_dir / "CERTS"

    try:
        data = scope.read_text()
        cookies = [ScopeCookie.parse(c) for c in ScopeCookie.split(data)]
        print("\n\n".join([str(c) for c in cookies]))

        if server is not None:
            certdata = certificates.read_text()
            for cookie in cookies:
                cookie.verify([server], certdata)
            print("Cookies verified successfully")
    except ScopeError as exc:
        print(exc)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['opendiamond', 'opendiamond.console']

package_data = \
{'': ['*']}

install_requires = \
['M2Crypto>=0.21.1', 'click>=8.0.3,<9.0.0', 'python-dateutil>=1.5']

entry_points = \
{'console_scripts': ['cookiecutter = '
                     'opendiamond.console.scope_generate:generate',
                     'diamond-newscope = '
                     'opendiamond.console.scope_import:import_',
                     'opendiamond-scope = opendiamond.console.scope:cli'],
 'opendiamond.cli_plugins': ['scope = opendiamond.console.scope:cli']}

setup_kwargs = {
    'name': 'opendiamond-scope',
    'version': '10.1.5',
    'description': 'OpenDiamond scope manipulation library and tools',
    'long_description': '# OpenDiamond-scope\n\nLibrary and tools for manipulating OpenDiamond search scopes.\n\n\n# To install the latest release from PyPI\n\n    pipx install opendiamond-scope          # or\n    pip install --user opendiamond-scope\n\n\n# Building from source\n\nThe build depends on SWIG and OpenSSL to build the M2Crypto dependency, the\nfollowing command will install this on a Debian/Ubuntu based system.\n\n    sudo apt install swig libssl-dev\n\nFor development, a consistent development environment is managed with poetry.\nIf you are developing locally it is recommended to set up pre-commit git hooks.\n\n    poetry install\n    poetry run pre-commit install   # optional\n\nThe code can then be run from the managed environment.\n\n    poetry run opendiamond-scope -h\n\nRunning of tests and release tagging is done with nox, which should already be\ninstalled in the development environment at this point.\n\n    # run tests against different installed python interpreters\n    poetry run nox\n\n    # release version tagging (and publishing) are handled by a nox script\n    poetry run nox -s release -- [major/minor/patch]\n',
    'author': 'Carnegie Mellon University',
    'author_email': 'diamond@cs.cmu.edu',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://diamond.cs.cmu.edu',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6.1,<4.0',
}


setup(**setup_kwargs)



#
#  The OpenDiamond Platform for Interactive Search
#
# SPDX-License-Identifier: EPL-1.0
#
#  Copyright (c) 2009-2021 Carnegie Mellon University
#  All rights reserved.
#
#  This software is distributed under the terms of the Eclipse Public
#  License, Version 1.0 which can be found in LICENSES/EPL-1.0.
#  ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS SOFTWARE CONSTITUTES
#  RECIPIENT'S ACCEPTANCE OF THIS AGREEMENT
#
"""CLI application to generate a new OpenDiamond scope"""


import os
import sys
from datetime import datetime, timedelta

import click
from dateutil.tz import tzutc

from ..scope import ScopeCookie
from .scope_options import CONTEXT_SETTINGS


@click.command(context_settings=CONTEXT_SETTINGS)
@click.option("-v", "--verbose", is_flag=True, help="be verbose")
@click.option(
    "-b",
    "--blaster",
    metavar="URL",
    help="JSON Blaster that can relay for the specified servers",
)
@click.option(
    "-s",
    "--server",
    "servers",
    metavar="HOST",
    multiple=True,
    required=True,
    help="server that accepts the cookie (can be repeated)",
)
@click.option(
    "-e",
    "--expires",
    metavar="TIME",
    default=3600,
    help="time in seconds until cookie expires (3600)",
)
@click.option(
    "-k",
    "--keyfile",
    type=click.File("rb"),
    help="X509 private key ($HOME/.diamond/key.pem)",
)
@click.option(
    "-u",
    "--scopeurl",
    "scopeurls",
    metavar="URL",
    multiple=True,
    help="URL from which scopelist can be retrieved (can be repeated)",
)
# pylint: disable=too-many-arguments
def generate(verbose, blaster, servers, expires, keyfile, scopeurls):
    """Generates an OpenDiamond(R) scope cookie.

    If no --scopeurl option is given, the list of scope URLs will be read
    from stdin.
    """
    if keyfile is None:
        app_dir = click.get_app_dir("diamond", force_posix=True)
        key_pem = os.path.join(app_dir, "key.pem")
        keyfile = click.open_file(key_pem, "rb")

    # Read private key
    with keyfile as fhandle:
        keydata = fhandle.read()

    # Gather scope data
    if len(scopeurls) == 0:
        scopeurl = [u.strip() for u in sys.stdin]

    # Calculate cookie expiration time
    expires = datetime.now(tzutc()) + timedelta(seconds=expires)

    # Build and sign the cookie
    cookie = ScopeCookie.generate(servers, scopeurl, expires, keydata, blaster=blaster)

    # Print decoded cookie to stderr if verbose
    if verbose:
        click.echo(str(cookie), err=True)

    # Print final cookie to stdout
    print(cookie.encode(), end=" ")



#
#  The OpenDiamond Platform for Interactive Search
#
# SPDX-License-Identifier: EPL-1.0
#
#  Copyright (c) 2021 Carnegie Mellon University
#  All rights reserved.
#
#  This software is distributed under the terms of the Eclipse Public
#  License, Version 1.0 which can be found in LICENSES/EPL-1.0.
#  ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS SOFTWARE CONSTITUTES
#  RECIPIENT'S ACCEPTANCE OF THIS AGREEMENT
#
"""Common arguments and options for opendiamond CLI applications"""

import pathlib

import click

# use with '@click.command(context_settings=CONTEXT_SETTINGS)'
CONTEXT_SETTINGS = dict(help_option_names=["-h", "--help"])

# installation path prefix
prefix_option = click.option(
    "--prefix",
    default=pathlib.Path.home() / ".local",
    type=click.Path(file_okay=False, path_type=pathlib.Path),
    show_default=True,
    help="installation prefix (/usr, /usr/local)",
)
